<?php declare(strict_types=1);
/*
 * This file is part of PHPUnit.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace PHPUnit\TestFixture\TransformExceptionHookMethod;

use PHPUnit\Framework\TestCase;
use Throwable;

final class Test extends TestCase
{
    public function testOne(): void
    {
        throw new OriginalException('original message');
    }

    protected function transformException(Throwable $t): Throwable
    {
        return new TransformedException('transformed message');
    }
}
