<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Seld\\PharUtils\\' => array($vendorDir . '/seld/phar-utils/src'),
);
