<?php return array(
    'root' => array(
        'name' => '__root__',
        'pretty_version' => '8.5.x-dev',
        'version' => '8.5.9999999.9999999-dev',
        'reference' => 'aca96fcd8b6799ead1066524b2b91c5184ece78f',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        '__root__' => array(
            'pretty_version' => '8.5.x-dev',
            'version' => '8.5.9999999.9999999-dev',
            'reference' => 'aca96fcd8b6799ead1066524b2b91c5184ece78f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'seld/phar-utils' => array(
            'pretty_version' => '1.2.1',
            'version' => '1.2.1.0',
            'reference' => 'ea2f4014f163c1be4c601b9b7bd6af81ba8d701c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../seld/phar-utils',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
