<?php declare(strict_types=1);
/*
 * This file is part of PHPUnit.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace PHPUnit\TestFixture;

use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\TestCase;

final class DataProviderWithStringKeysTest extends TestCase
{
    public static function providerMethod(): array
    {
        return [
            '0 + 0 = 0' => [0, 0, 0],
            '0 + 1 = 1' => [0, 1, 1],
            '1 + 1 = 3' => [1, 1, 3],
            '1 + 0 = 1' => [1, 0, 1],
        ];
    }

    #[DataProvider('providerMethod')]
    public function testAdd($a, $b, $c): void
    {
        $this->assertEquals($c, $a + $b);
    }
}
